﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Diagrams.Core;

namespace SilverlightApplication4
{
    public partial class MainPage : UserControl
    {
        const int ROOT_X_Position = 400;
        const int ROOT_Y_Position = 20;
        const int X_DELIMITER_WIDTH = 80;
        const int Y_DELIMITER_HEIGHT = 60;
        const int X_START_NEGATIVE_OFFSET = 40;

        public MainPage()
        {
            InitializeComponent();
            this.diagram.GraphSource = CreatePascalTriangleGraphSource(7);
        }



        private IGraphSource CreatePascalTriangleGraphSource(int levels)
        {
            PascalTriangleGraphSource graph = new PascalTriangleGraphSource();
            for (int i = 0; i < levels; i++)
            {
                for (int j = 0; j < i + 1; j++)
                {
                    PascalNode node = new PascalNode()
                    {
                        ToolTip =(i*j).ToString(),
                        Position = new Point()
                        {
                            Y = ROOT_Y_Position + i * Y_DELIMITER_HEIGHT,
                            X = ROOT_X_Position - i * X_START_NEGATIVE_OFFSET + j * X_DELIMITER_WIDTH
                        },
                        PascalNumber = Binom(i, j)
                    };
                    graph.InternalItems.Add(node);
                    int currIndex = i * (i + 1) / 2 + j;
                    if (j == 0) graph.InternalEdges.Add(new PascalEdge() { Source = graph.InternalItems[currIndex - i], Target = node });
                    if (j == i && i != 0) graph.InternalEdges.Add(new PascalEdge() { Source = graph.InternalItems[currIndex - i - 1], Target = node });
                    if (0 < j && j < i)
                    {
                        graph.InternalEdges.Add(new PascalEdge() { Source = graph.InternalItems[currIndex - i - 1], Target = node });
                        graph.InternalEdges.Add(new PascalEdge() { Source = graph.InternalItems[currIndex - i], Target = node });
                    }
                }
            }
            return graph;
        }

        private int Binom(int n, int k)
        {
            return FactN(n) / (FactN(k) * FactN(n - k));
        }

        private int FactN(int n)
        {
            if (n == 0 || n == 1) return 1;
            int res = 1;
            for (int i = 1; i <= n; i++)
            {
                res = res * i;
            }
            return res;
        }
    }
}
